// ../packages/cli/core/index.ts
import * as process2 from "process";
import { log as log4 } from "@pr-checker/utils";
import { cac } from "cac";
import { getUserName } from "@pr-checker/utils/git-api";

// ../package.json
var version = "1.3.0";

// ../packages/cli/core/store/storage.ts
import { existsSync, promises as fs, mkdirSync } from "fs";
import { resolve } from "path";
import { homedir } from "os";
var storage;
var storageDir = (() => {
  const home = homedir();
  const oldPath = resolve(home, "pr-checker-storage");
  if (existsSync(oldPath))
    return oldPath;
  const dotPath = resolve(home, ".config/pr-checker-storage");
  if (!existsSync(dotPath)) {
    mkdirSync(dotPath, {
      recursive: true
    });
  }
  return dotPath;
})();
var storagePath = resolve(storageDir, "_pr_checker_storage.json");
async function loadStorage(fn) {
  if (!storage) {
    storage = existsSync(storagePath) ? JSON.parse(await fs.readFile(storagePath, "utf-8") || "{}") || {} : {};
  }
  if (fn) {
    if (await fn(storage))
      await saveStorage();
  }
  return storage;
}
async function saveStorage() {
  if (storage) {
    if (!existsSync(storageDir))
      await fs.mkdir(storageDir, { recursive: true });
    await fs.writeFile(storagePath, JSON.stringify(storage), "utf-8");
  }
}
async function clearStorage() {
  if (storage) {
    if (!existsSync(storageDir))
      await fs.mkdir(storageDir, { recursive: true });
    await fs.writeFile(storagePath, "{}", "utf-8");
  }
}

// ../packages/cli/core/select/handle-select.ts
import ora from "ora";
import { GitApi, createRunList, isEmptyObj, log as log2 } from "@pr-checker/utils";
import { Table } from "console-table-printer";
import chalk2 from "chalk";

// ../packages/cli/core/select/select-configure.ts
import chalk from "chalk";
import { formatEllipsis, log } from "@pr-checker/utils";
import prompts from "prompts";
var typeOption = [{
  type: "select",
  name: "typeSelect",
  message: "Detect all Repo's PR?",
  choices: ["All Repo", "Detect the PR of a certain Repo"].map((item) => {
    let value = "all";
    if (item !== "All Repo")
      value = "select";
    return { title: item, value };
  })
}];
var createRepoOption = (list) => {
  const handler = (item) => {
    return { title: item, value: item };
  };
  return [{
    type: "select",
    name: "RepoSelect",
    message: "Please select a Repo",
    choices: list.map(handler)
  }];
};
var createPrOption = (list) => {
  const handler = (item) => {
    const repo = chalk.blueBright.bold(`[${formatEllipsis(item.repo)}]`);
    const number = chalk.yellowBright.bold(`[#${item.number}]`);
    const title = formatEllipsis(`${item.title}`, 36);
    const canMerge = item.isNeedUpdate ? chalk.yellowBright.bold("<can merge>") : chalk.redBright.bold(`<can\`t merge:${item.reason}>`);
    const infoTitle = `${canMerge}: ${repo}-${number} -> ${title}`;
    return {
      title: infoTitle,
      value: {
        title: item.title,
        number: item.number,
        repo: item.repo,
        isNeedUpdate: item.isNeedUpdate,
        reason: item.reason,
        infoTitle
      }
    };
  };
  return [{
    type: "multiselect",
    name: "prSelect",
    message: "Please select the PR that needs to be updated",
    optionsPerPage: 20,
    choices: list.map(handler),
    onRender() {
      if (this.firstRender) {
        this.renderDoneOrInstructions = function() {
          return "";
        };
      }
    }
  }];
};
var promptsRun = async (option) => {
  const res = await prompts(option, {
    onCancel: () => {
      log("info", "Operation cancel");
      process.exit();
    }
  });
  return { ...res };
};

// ../packages/cli/core/select/handle-select.ts
async function handleSelect(store, mode) {
  const isAllRepo = await promptsRun(typeOption);
  let updateRes = [];
  const spinner = ora({ text: "Loading Repo......", color: "blue" }).start();
  const githubApi = new GitApi(store.token, store.username);
  const prList = mode === "rebase" ? await githubApi.getSubmitPRList() : await githubApi.getAllRepoPRList();
  spinner.stop();
  const repoList = Object.keys(prList);
  if (isAllRepo.typeSelect === "select") {
    const selectRepo = await promptsRun(createRepoOption(repoList));
    const prl = prList[selectRepo.RepoSelect];
    log2("info", `Checking PR by ${selectRepo.RepoSelect}......`);
    const prListByRepo = await checkPR(prl, githubApi, mode);
    const prSelectRes = await promptsRun(createPrOption(prListByRepo));
    log2("info", `Update PR by ${selectRepo.RepoSelect}......`);
    updateRes = await updatePR(prl, prSelectRes, githubApi, mode);
  } else {
    const prl = [];
    repoList.forEach((val) => {
      prList[val].forEach((item) => {
        prl.push(item);
      });
    });
    log2("info", "Checking PR......");
    const prListByRepo = await checkPR(prl, githubApi, mode);
    const prSelectRes = await promptsRun(createPrOption(prListByRepo));
    log2("info", "Update PR......");
    updateRes = await updatePR(prl, prSelectRes, githubApi, mode);
  }
  spinner.succeed();
  log2("success", "\u2714 All PR updates completed");
  await printUpdateRes(updateRes);
}
async function checkPR(prl, githubApi, mode) {
  if (prl.length === 0) {
    log2("error", "Please select a pr to check");
    process.exit();
  }
  const prListByRepo = await Promise.all(createRunList(prl.length, async (i) => {
    const prInfo = await githubApi.getPRByRepo(prl[i].number, prl[i].repo, prl[i].title);
    const res = await githubApi.needUpdate(prl[i].repo, prInfo, mode);
    log2("success", `\u2714 Check PR #${prl[i].number} completed`);
    return {
      ...prInfo,
      ...res
    };
  }));
  return prListByRepo;
}
async function updatePR(prl, prSelectRes, githubApi, mode) {
  const updateRes = await Promise.all(createRunList(prl.length, async (i) => {
    if (prSelectRes.prSelect[i] && prSelectRes.prSelect[i].isNeedUpdate) {
      if (mode === "rebase")
        await githubApi.rebasePR(prSelectRes.prSelect[i].number, prSelectRes.prSelect[i].repo);
      if (mode === "merge")
        await githubApi.mergePR(prSelectRes.prSelect[i].number, prSelectRes.prSelect[i].repo);
      log2("success", `\u2714 Update PR #${prl[i].number} completed`);
    }
    return {
      ...prSelectRes.prSelect[i]
    };
  }));
  return updateRes;
}
async function printUpdateRes(res) {
  const p = new Table({
    columns: [
      { name: "number", alignment: "left" },
      { name: "can merge", alignment: "left" },
      { name: "success", alignment: "left" },
      { name: "reason", alignment: "left" },
      { name: "repo", alignment: "left" },
      { name: "title", alignment: "left" }
    ]
  });
  res.sort((a, b) => Number(a.isNeedUpdate) - Number(b.isNeedUpdate));
  res.forEach((item) => {
    if (!isEmptyObj(item)) {
      p.addRow({
        "number": chalk2.greenBright.bold(`#${item.number}`),
        "can merge": item.isNeedUpdate ? chalk2.greenBright.bold("true") : chalk2.redBright.bold("false"),
        "success": item.isNeedUpdate ? chalk2.greenBright.bold("true") : chalk2.redBright.bold("false"),
        "reason": item.isNeedUpdate ? chalk2.blueBright.bold("-") : item.reason === "not updated" ? chalk2.blueBright.bold(item.reason) : chalk2.yellowBright.bold(item.reason),
        "repo": chalk2.blueBright.bold(`<${item.repo}>`),
        "title": item.title
      });
    }
  });
  p.printTable();
}

// ../packages/cli/core/option/handle-option.ts
import { log as log3 } from "@pr-checker/utils";
async function handleOption(parseRes) {
  const storage2 = await loadStorage();
  const {
    u,
    username,
    t,
    token,
    c,
    clear,
    g,
    get
  } = parseRes.options;
  if (c || clear) {
    if (storage2) {
      await clearStorage();
      log3("success", "clear token and username successfully");
      return;
    }
  }
  if (g || get) {
    if (storage2) {
      log3("info", `username: ${storage2.username || "null"}`);
      log3("info", `token: ${storage2.token || "null"}`);
      return;
    }
  }
  if (t && typeof t === "string" || token && typeof token === "string") {
    storage2.token = t || token;
    await saveStorage();
    log3("success", "token set successfully");
    return;
  }
  if (u && typeof u === "string" || username && typeof username === "string") {
    storage2.username = u || username;
    await saveStorage();
    log3("success", "username set successfully");
  }
}

// ../packages/cli/core/index.ts
var run = async () => {
  const cliInst = await initCli();
  await handleOption(cliInst.parse());
};
async function initCli() {
  const cli = cac("pr-checker");
  const storage2 = await loadStorage();
  cli.option("-t <token>,--token <token>", "set github token");
  cli.option("-u <username>, --username <username>", "set github username");
  cli.option("-c, --clear", "clear token and username");
  cli.option("-g, --get", "get git config");
  cli.command("run", "check your pr").option("-m <value>, --mode <value>", "use rebase or merge").action(async (options) => {
    if (!storage2.token) {
      log4("error", "use `pr-checker -t <TOKEN>` to set your token");
      process2.exit(1);
    }
    if (!storage2.username) {
      log4("info", "You have not set a username, it has been automatically set for you according to the token");
      const { login } = await getUserName(storage2.token);
      storage2.username = login;
      await saveStorage();
    }
    const { m, mode } = options;
    const lastMode = m || mode || "rebase";
    await handleSelect(storage2, lastMode);
  });
  cli.help();
  cli.version(version);
  return cli;
}
export {
  run
};
