// ../utils/log.ts
import chalk from "chalk";
var log = (type, msg) => {
  if (type === "info")
    console.log(chalk.blueBright.bold(`[pr-checker]:${msg}`));
  if (type === "error")
    console.log(chalk.redBright.bold(`[pr-checker]:${msg}`));
  if (type === "warning")
    console.log(chalk.yellowBright.bold(`[pr-checker]:${msg}`));
  if (type === "success")
    console.log(chalk.greenBright.bold(`[pr-checker]:${msg}`));
};

// ../utils/git-api.ts
import { Octokit } from "@octokit/core";
import { createRunList, isEmptyObj, log as log2 } from "@pr-checker/utils";
var GitApi = class {
  octokit;
  owner;
  constructor(token, owner) {
    this.octokit = new Octokit({
      auth: token
    });
    this.owner = owner;
  }
  /**
   * 获取账户下所有提交 pr
   * @param username
   */
  async getSubmitPRList(username = this.owner) {
    try {
      const { data } = await this.octokit.request("GET /search/issues", {
        q: `is:pr is:open author:${username}`,
        per_page: 1e3
      });
      if (!data.items || data.items && data.items.length === 0)
        log2("error", "You don't have any pull requests that are open");
      const res = {};
      data.items.forEach((val) => {
        const repo = val.repository_url.split("repos/")[1];
        if (!res[repo])
          res[repo] = [];
        res[repo].push({
          title: val.title,
          number: val.number,
          repo,
          id: val.id
        });
      });
      return res;
    } catch (error) {
      if (error.status === 401)
        log2("error", "Your token is invalid or does not match your username");
      log2("error", error);
      return {};
    }
  }
  async getAllRepoPRList(username = this.owner) {
    try {
      const { data } = await this.octokit.request("GET /user/repos", {
        type: "owner",
        per_page: 1e3
      });
      const notForkRepoList = data.filter((v) => !v.fork);
      const res = {};
      await Promise.all(createRunList(notForkRepoList.length, async (i) => {
        const { data: prData } = await this.octokit.request(
          "GET /repos/{owner}/{repo}/pulls",
          {
            owner: username,
            repo: notForkRepoList[i].name,
            per_page: 1e3
          }
        );
        prData.forEach((val) => {
          const repo = notForkRepoList[i].full_name;
          if (!res[repo])
            res[repo] = [];
          res[repo].push({
            title: val.title,
            number: val.number,
            repo,
            id: val.id
          });
        });
      }));
      if (isEmptyObj(res))
        log2("error", "You don't have any pull requests");
      return res;
    } catch (error) {
      if (error.status === 401)
        log2("error", "Your token is invalid or does not match your username");
      log2("error", error);
      return {};
    }
  }
  /**
   * Get pr details according to pr number and repo name
   * @param pull_number pr 号
   * @param repo_name upstream Repo name
   * @param title
   */
  async getPRByRepo(pull_number, repo_name, title) {
    try {
      const { data } = await this.octokit.request(
        `GET /repos/${repo_name}/pulls/{pull_number}`,
        {
          pull_number
        }
      );
      return {
        sha: data.base.sha,
        // Fork warehouse default branch hash
        org_repo: data.base.repo.full_name,
        org_repo_default_branch: data.base.repo.default_branch,
        pr_sha: data.head.sha,
        pr_repo: data.head.repo.default_branch,
        pr_repo_default_branch: data.head.repo.default_branch,
        merged: data.merged,
        mergeable: data.mergeable,
        mergeable_state: data.mergeable_state,
        number: pull_number,
        repo: repo_name,
        title: title || ""
      };
    } catch (error) {
      log2("error", error);
      return {};
    }
  }
  /**
   * Determine whether pr wants to synchronize the upstream Repo
   * @param repo_name upstream Repo name
   * @param pr_info The getPRByRepo function returns the result
   * @param mode
   */
  async needUpdate(repo_name, pr_info, mode) {
    try {
      if (pr_info.mergeable_state === "dirty")
        return { isNeedUpdate: false, reason: "code conflict" };
      const { data } = await this.octokit.request(
        `GET /repos/${repo_name}/commits`
      );
      if (data.length >= 0) {
        const cur_sha = data[0].sha;
        const matchedSha = mode === "rebase" && pr_info.sha !== cur_sha || mode === "merge";
        if (matchedSha && pr_info.mergeable && !pr_info.merged)
          return { isNeedUpdate: true, reason: "--" };
        else
          return { isNeedUpdate: false, reason: "not updated" };
      } else {
        return { isNeedUpdate: false, reason: "unknown error" };
      }
    } catch (error) {
      log2("error", error);
      return { isNeedUpdate: false, reason: "unknown error" };
    }
  }
  /**
   * update pr
   * @param pull_number
   * @param repo_name
   */
  async rebasePR(pull_number, repo_name) {
    try {
      const res = await this.octokit.request(
        `PUT /repos/${repo_name}/pulls/{pull_number}/update-branch`,
        {
          pull_number
        }
      );
      return res.data.message;
    } catch (error) {
      log2("error", error);
      return {};
    }
  }
  async mergePR(pull_number, repo_name) {
    try {
      const res = await this.octokit.request(
        `PUT /repos/${repo_name}/pulls/{pull_number}/merge`,
        {
          pull_number
        }
      );
      return res.data.message;
    } catch (error) {
      log2("error", error);
      return {};
    }
  }
};
async function getUserName(token) {
  const octokit = new Octokit({
    auth: token
  });
  const { data } = await octokit.request("GET /user");
  return data;
}

// ../utils/index.ts
function createRunList2(taskNum, taskFunc) {
  const taskList = [];
  for (let i = 0; i < taskNum; i++) {
    taskList.push(new Promise((resolve) => {
      resolve(taskFunc(i));
    }));
  }
  return taskList;
}
var isEmptyObj2 = (item) => JSON.stringify(item) === "{}";
function formatEllipsis(str = "", limitLen = 24) {
  let len = 0;
  const reg = /[\x00-\xFF]/;
  const strContent = str.split("");
  const inx = strContent.findIndex((s) => {
    len += reg.test(s) ? 1 : 2;
    if (len > limitLen)
      return true;
    else
      return false;
  });
  return inx === -1 ? str : `${str.substr(0, inx)}...`;
}
export {
  GitApi,
  createRunList2 as createRunList,
  formatEllipsis,
  getUserName,
  isEmptyObj2 as isEmptyObj,
  log
};
